package com.dragonsoft.dcuc.approve.event.listener;

import com.dragoninfo.dcuc.duceap.enums.OperateDisposeTypeEnum;
import com.dragonsoft.auditlog.collection.qmtj.LogSendComponent;
import com.dragonsoft.auditlog.collection.qmtj.pojo.req.ApproveBusLog;
import com.dragonsoft.dcuc.approve.enumresources.QmApproveTypeEnum;
import com.dragonsoft.dcuc.approve.event.event.AuditApproveOperateLogEvent;
import com.dragonsoft.dcuc.approve.model.dto.QmOperateLogDTO;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * @author wangrs
 * @date 2021/01/22
 * @desc 事件监听类
 */
@Component
public class ApproveOperateLogEventListener implements ApplicationListener<AuditApproveOperateLogEvent> {

    private LogSendComponent logSendComponent;

    private ApproveProperties approveProperties;

    @Autowired
    public void setLogSendComponent(LogSendComponent logSendComponent) {
        this.logSendComponent = logSendComponent;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    /**
     * 审批操作日志
     */
    public static final String OPERATE_LOG_TYPE = "009";

    /**
     * 是否扩线，就是关联审批流程
     */
    public static final String EXTEND = "1";

    @Override
    public void onApplicationEvent(AuditApproveOperateLogEvent event) {
        if (!approveProperties.getSendToAudit()) {
            return;
        }

        String auditSysId = approveProperties.getAuditSysId();

        QmOperateLogDTO qmOperateLogDTO = event.getQmOperateLogDTO();
        String appToken = qmOperateLogDTO.getAppToken();
        String requestIdcard = qmOperateLogDTO.getRequestIdcard();
        String requestUsername = qmOperateLogDTO.getRequestUsername();
        String requestApproveIp = qmOperateLogDTO.getRequestApproveIp();

        QmApproveTypeEnum qmApproveTypeEnum = qmOperateLogDTO.getQmApproveTypeEnum();
        String businessType = qmOperateLogDTO.getBusinessType();
        String approveIdcard = qmOperateLogDTO.getApproveIdcard();
        String approveUserName = qmOperateLogDTO.getApproveUserName();
        OperateDisposeTypeEnum operateDisposeTypeEnum = qmOperateLogDTO.getOperateDisposeTypeEnum();
        Date approveTime = qmOperateLogDTO.getApproveTime();
        // 文档有误，实际需使用毫秒
        String approveTimeString = approveTime.getTime() + "";
        String approveLogId = qmOperateLogDTO.getApproveLogId();

        Boolean flowFinish = qmOperateLogDTO.getFlowFinish();
        String approveNo = qmOperateLogDTO.getApproveNo();

        ApproveBusLog approveBusLog = new ApproveBusLog();
        approveBusLog.setAppToken(appToken);
        approveBusLog.setApplyApproveId(requestIdcard);
        approveBusLog.setApplyApproveIdCard(requestIdcard);
        approveBusLog.setApplyApproveName(requestUsername);
        approveBusLog.setApplyApproveIp(requestApproveIp);
        approveBusLog.setOperateTime(approveTimeString);
        approveBusLog.setApplyType(qmApproveTypeEnum.getValue());
        approveBusLog.setBusinessType(businessType);
        approveBusLog.setApproveContentHash("");
        approveBusLog.setApproveTableHash("");

        if (qmApproveTypeEnum.equals(QmApproveTypeEnum.OPERATE)) {
            approveBusLog.setApproveId(approveIdcard);
            approveBusLog.setApproveIdCard(approveIdcard);
            approveBusLog.setApproveName(approveUserName);
            approveBusLog.setApproveInforceTime(approveTimeString);

            if (flowFinish != null && flowFinish) {
                approveBusLog.setApproveResult(operateDisposeTypeEnum.getAuditCode());
                approveBusLog.setApproveTime(approveTimeString);
            }
        }
        approveBusLog.setApproveTaskId(approveLogId);
        approveBusLog.setApplySystemId("");
        approveBusLog.setApproveCenterId(approveNo);
        approveBusLog.setExtend(EXTEND);
        approveBusLog.setRelatedId("");

        List<ApproveBusLog> approveBusLogList = Collections.singletonList(approveBusLog);
        logSendComponent.sendApproveBusLog(auditSysId, OPERATE_LOG_TYPE, approveBusLogList);
    }
}
